@@ -39,13 +39,14 @@
 
 #include "ecc.h"
 #include "ecc-internal.h"
+#include "nettle-internal.h"
 
 void
 ecc_mul_g (const struct ecc_curve *ecc, mp_limb_t *r,
 	   const mp_limb_t *np, mp_limb_t *scratch)
 {
   /* Scratch need determined by the ecc_add_jja call. Current total is
-     9 * ecc->p.size, at most 648 bytes. */
+     8 * ecc->p.size, at most 576 bytes. */
 #define tp scratch
 #define scratch_out (scratch + 3*ecc->p.size)
 
@@ -71,7 +72,8 @@ ecc_mul_g (const struct ecc_curve *ecc, mp_limb_t *r,
 	  /* Avoid the mp_bitcnt_t type for compatibility with older GMP
 	     versions. */
 	  unsigned bit_index;
-	  
+	  int bits_is_zero;
+
 	  /* Extract c bits from n, stride k, starting at i + kcj,
 	     ending at i + k (cj + c - 1)*/
 	  for (bits = 0, bit_index = i + k*(c*j+c); bit_index > i + k*c*j; )
@@ -88,18 +90,20 @@ ecc_mul_g (const struct ecc_curve *ecc, mp_limb_t *r,
 	      shift = bit_index % GMP_NUMB_BITS;
 	      bits = (bits << 1) | ((np[limb_index] >> shift) & 1);
 	    }
-	  sec_tabselect (tp, 2*ecc->p.size,
-			 (ecc->pippenger_table
-			  + (2*ecc->p.size * (mp_size_t) j << c)),
-			 1<<c, bits);
+	  mpn_sec_tabselect (tp,
+			     (ecc->pippenger_table
+			      + (2*ecc->p.size * (mp_size_t) j << c)),
+			      2*ecc->p.size, 1<<c, bits);
 	  cnd_copy (is_zero, r, tp, 2*ecc->p.size);
 	  cnd_copy (is_zero, r + 2*ecc->p.size, ecc->unit, ecc->p.size);
 	  
 	  ecc_add_jja (ecc, tp, r, tp, scratch_out);
+	  bits_is_zero = IS_ZERO_SMALL (bits);
+
 	  /* Use the sum when valid. ecc_add_jja produced garbage if
-	     is_zero != 0 or bits == 0, . */	  
-	  cnd_copy (bits & (is_zero - 1), r, tp, 3*ecc->p.size);
-	  is_zero &= (bits == 0);
+	     is_zero or bits_is_zero. */
+	  cnd_copy (1 - (bits_is_zero | is_zero), r, tp, 3*ecc->p.size);
+	  is_zero &= bits_is_zero;
 	}
     }
 #undef tp
