@@ -42,57 +42,41 @@
 #include "nettle-internal.h"
 
 static int
-zero_p (const struct ecc_curve *ecc,
-	const mp_limb_t *xp)
-{
-  mp_limb_t t;
-  mp_size_t i;
-
-  for (i = t = 0; i < ecc->size; i++)
-    t |= xp[i];
-
-  return t == 0;
-}
-
-static int
-ecdsa_in_range (const struct ecc_curve *ecc,
+ecdsa_in_range (const struct ecc_modulo *m,
 		const mp_limb_t *xp, mp_limb_t *scratch)
 {
   /* Check if 0 < x < q, with data independent timing. */
-  return !zero_p (ecc, xp)
-    & (mpn_sub_n (scratch, xp, ecc->q, ecc->size) != 0);
+  return !sec_zero_p (xp, m->size)
+    & (mpn_sub_n (scratch, xp, m->m, m->size) != 0);
 }
 
 void
-ecc_modq_random (const struct ecc_curve *ecc, mp_limb_t *xp,
-		 void *ctx, nettle_random_func *random, mp_limb_t *scratch)
+ecc_mod_random (const struct ecc_modulo *m, mp_limb_t *xp,
+		void *ctx, nettle_random_func *random, mp_limb_t *scratch)
 {
   uint8_t *buf = (uint8_t *) scratch;
-  unsigned nbytes = (ecc->bit_size + 7)/8;
+  unsigned nbytes = (m->bit_size + 7)/8;
 
   /* The bytes ought to fit in the scratch area, unless we have very
      unusual limb and byte sizes. */
-  assert (nbytes <= ecc->size * sizeof (mp_limb_t));
+  assert (nbytes <= m->size * sizeof (mp_limb_t));
 
   do
     {
-      /* q and p are of the same bitsize. */
       random (ctx, nbytes, buf);
-      buf[0] &= 0xff >> (nbytes * 8 - ecc->bit_size);
+      buf[0] &= 0xff >> (nbytes * 8 - m->bit_size);
 
-      mpn_set_base256 (xp, ecc->size, buf, nbytes);
+      mpn_set_base256 (xp, m->size, buf, nbytes);
     }
-  while (!ecdsa_in_range (ecc, xp, scratch));
+  while (!ecdsa_in_range (m, xp, scratch));
 }
 
 void
 ecc_scalar_random (struct ecc_scalar *x,
 		   void *random_ctx, nettle_random_func *random)
 {
-  TMP_DECL (scratch, mp_limb_t, ECC_MODQ_RANDOM_ITCH (ECC_MAX_SIZE));
-  TMP_ALLOC (scratch, ECC_MODQ_RANDOM_ITCH (x->ecc->size));
+  TMP_DECL (scratch, mp_limb_t, ECC_MOD_RANDOM_ITCH (ECC_MAX_SIZE));
+  TMP_ALLOC (scratch, ECC_MOD_RANDOM_ITCH (x->ecc->q.size));
 
-  ecc_modq_random (x->ecc, x->p, random_ctx, random, scratch);
+  ecc_mod_random (&x->ecc->q, x->p, random_ctx, random, scratch);
 }
-
-
