@@ -49,6 +49,7 @@
 #include "cbc.h"
 #include "hmac.h"
 #include "macros.h"
+#include "memops.h"
 #include "rsa.h"
 #include "yarrow.h"
 
@@ -63,7 +64,7 @@ rsa_session_set_decrypt_key(struct rsa_session *ctx,
   const uint8_t *iv = SESSION_IV(key);
   const uint8_t *hmac_key = SESSION_HMAC_KEY(key);
   
-  aes_set_decrypt_key(&ctx->aes.ctx, AES_KEY_SIZE, aes_key);
+  aes256_set_decrypt_key(&ctx->aes.ctx, aes_key);
   CBC_SET_IV(&ctx->aes, iv);
   hmac_sha1_set_key(&ctx->hmac, SHA1_DIGEST_SIZE, hmac_key);
 }
@@ -108,13 +109,6 @@ read_bignum(FILE *f, mpz_t x)
   return 0;
 }
 
-struct process_ctx
-{
-  struct CBC_CTX(struct aes_ctx, AES_BLOCK_SIZE) aes;
-  struct hmac_sha1_ctx hmac;
-  struct yarrow256_ctx yarrow;
-};
-
 #define BUF_SIZE (100 * AES_BLOCK_SIZE)
 
 /* Trailing data that needs special processing */
@@ -157,9 +151,9 @@ process_file(struct rsa_session *ctx,
 
       if (size)
 	{
-	  CBC_DECRYPT(&ctx->aes, aes_decrypt, size, buffer, buffer);
+	  CBC_DECRYPT(&ctx->aes, aes256_decrypt, size, buffer, buffer);
 	  hmac_sha1_update(&ctx->hmac, size, buffer);
-	  if (!write_string(out, size, buffer))
+	  if (!write_data(out, size, buffer))
 	    {
 	      werror("Writing output failed: %s\n", strerror(errno));
 	      return 0;
@@ -170,7 +164,7 @@ process_file(struct rsa_session *ctx,
   while (size == BUF_SIZE);
 
   /* Decrypt final block */
-  CBC_DECRYPT(&ctx->aes, aes_decrypt, AES_BLOCK_SIZE, buffer, buffer);
+  CBC_DECRYPT(&ctx->aes, aes256_decrypt, AES_BLOCK_SIZE, buffer, buffer);
   padding = buffer[AES_BLOCK_SIZE - 1];
   if (padding > AES_BLOCK_SIZE)
     {
@@ -182,14 +176,14 @@ process_file(struct rsa_session *ctx,
     {
       unsigned leftover = AES_BLOCK_SIZE - padding;
       hmac_sha1_update(&ctx->hmac, leftover, buffer);
-      if (!write_string(out, leftover, buffer))
+      if (!write_data(out, leftover, buffer))
 	{
 	  werror("Writing output failed: %s\n", strerror(errno));
 	  return 0;
 	}
     }
   hmac_sha1_digest(&ctx->hmac, SHA1_DIGEST_SIZE, digest);
-  if (memcmp(digest, buffer + AES_BLOCK_SIZE, SHA1_DIGEST_SIZE) != 0)
+  if (!memeql_sec(digest, buffer + AES_BLOCK_SIZE, SHA1_DIGEST_SIZE))
     {
       werror("Decryption failed: Invalid mac.\n");
       return 0;
