@@ -1,26 +1,33 @@
 /* rsa-keygen.c
- *
- */
 
-/* nettle, low-level cryptographics library
- *
- * Copyright (C) 2002 Niels Möller
- *  
- * The nettle library is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or (at your
- * option) any later version.
- * 
- * The nettle library is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
- * License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public License
- * along with the nettle library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- * MA 02111-1301, USA.
- */
+   Copyright (C) 2002 Niels Möller
+
+   This file is part of GNU Nettle.
+
+   GNU Nettle is free software: you can redistribute it and/or
+   modify it under the terms of either:
+
+     * the GNU Lesser General Public License as published by the Free
+       Software Foundation; either version 3 of the License, or (at your
+       option) any later version.
+
+   or
+
+     * the GNU General Public License as published by the Free
+       Software Foundation; either version 2 of the License, or (at your
+       option) any later version.
+
+   or both in parallel, as here.
+
+   GNU Nettle is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   General Public License for more details.
+
+   You should have received copies of the GNU General Public License and
+   the GNU Lesser General Public License along with this program.  If
+   not, see http://www.gnu.org/licenses/.
+*/
 
 #if HAVE_CONFIG_H
 # include "config.h"
@@ -41,7 +48,7 @@
 
 #include "getopt.h"
 
-#define KEYSIZE 900
+#define DEFAULT_KEYSIZE 2048
 #define ESIZE 30
 
 static void
@@ -51,6 +58,22 @@ progress(void *ctx, int c)
   fputc(c, stderr);
 }
 
+static unsigned long
+uint_arg (char c, const char *arg)
+{
+  unsigned long val;
+  char *end;
+
+  val = strtoul(arg, &end, 0);
+  if (*arg == '\0' || *end != '\0')
+    {
+      werror ("Invalid integer argument for -%c option.\n", c);
+      exit (EXIT_FAILURE);
+    }
+
+  return val;      
+}
+
 int
 main(int argc, char **argv)
 {
@@ -66,6 +89,9 @@ main(int argc, char **argv)
   struct nettle_buffer pub_buffer;
   struct nettle_buffer priv_buffer;
 
+  unsigned long key_size = DEFAULT_KEYSIZE;
+  unsigned long key_e = 0;
+
   enum { OPT_HELP = 300 };
   static const struct option options[] =
     {
@@ -75,9 +101,9 @@ main(int argc, char **argv)
       { NULL, 0, NULL, 0}
     };
   
-  while ( (c = getopt_long(argc, argv, "o:r:", options, NULL)) != -1)
+  while ( (c = getopt_long(argc, argv, "o:r:e:s:", options, NULL)) != -1)
     switch (c)
-      {
+      {	
       case 'o':
 	priv_name = optarg;
 	break;
@@ -86,6 +112,14 @@ main(int argc, char **argv)
 	random_name = optarg;
 	break;
 
+      case 's':
+	key_size = uint_arg ('s', optarg);
+	break;
+
+      case 'e':
+	key_e = uint_arg ('e', optarg);
+	break;
+
       case OPT_HELP:
 	printf("FIXME: Usage information.\n");
 	return EXIT_SUCCESS;
@@ -119,11 +153,14 @@ main(int argc, char **argv)
   rsa_public_key_init(&pub);
   rsa_private_key_init(&priv);
 
+  if (key_e)
+    mpz_set_ui (pub.e, key_e);
+
   if (!rsa_generate_keypair
       (&pub, &priv,
        (void *) &yarrow, (nettle_random_func *) yarrow256_random,
        NULL, progress,
-       KEYSIZE, ESIZE))
+       key_size, key_e == 0 ? ESIZE : 0))
     {
       werror("Key generation failed.\n");
       return EXIT_FAILURE;
