@@ -1,32 +1,40 @@
 /* serpent-decrypt.c
- *
- * The serpent block cipher.
- *
- * For more details on this algorithm, see the Serpent website at
- * http://www.cl.cam.ac.uk/~rja14/serpent.html
- */
 
-/* nettle, low-level cryptographics library
- *
- * Copyright (C) 2011  Niels Möller
- * Copyright (C) 2010, 2011  Simon Josefsson
- * Copyright (C) 2003, 2004, 2005 Free Software Foundation, Inc.
- *  
- * The nettle library is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or (at your
- * option) any later version.
- * 
- * The nettle library is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
- * License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public License
- * along with the nettle library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
- * MA 02111-1307, USA.
- */
+   The serpent block cipher.
+
+   For more details on this algorithm, see the Serpent website at
+   http://www.cl.cam.ac.uk/~rja14/serpent.html
+
+   Copyright (C) 2011  Niels Möller
+   Copyright (C) 2010, 2011  Simon Josefsson
+   Copyright (C) 2003, 2004, 2005 Free Software Foundation, Inc.
+
+   This file is part of GNU Nettle.
+
+   GNU Nettle is free software: you can redistribute it and/or
+   modify it under the terms of either:
+
+     * the GNU Lesser General Public License as published by the Free
+       Software Foundation; either version 3 of the License, or (at your
+       option) any later version.
+
+   or
+
+     * the GNU General Public License as published by the Free
+       Software Foundation; either version 2 of the License, or (at your
+       option) any later version.
+
+   or both in parallel, as here.
+
+   GNU Nettle is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   General Public License for more details.
+
+   You should have received copies of the GNU General Public License and
+   the GNU Lesser General Public License along with this program.  If
+   not, see http://www.gnu.org/licenses/.
+*/
 
 /* This file is derived from cipher/serpent.c in Libgcrypt v1.4.6.
    The adaption to Nettle was made by Simon Josefsson on 2010-12-07
@@ -438,16 +446,16 @@
 /* In-place inverse linear transformation.  */
 #define LINEAR_TRANSFORMATION64_INVERSE(x0,x1,x2,x3)	 \
   do {                                                   \
-    x2 = ROTL64 (10, x2);                    \
-    x0 = ROTL64 (27, x0);                    \
-    x2 = x2 ^ x3 ^ RSHIFT64(7, x1); \
+    x2 = DROTL32 (10, x2);                    \
+    x0 = DROTL32 (27, x0);                    \
+    x2 = x2 ^ x3 ^ DRSHIFT32(7, x1); \
     x0 = x0 ^ x1 ^ x3;        \
-    x3 = ROTL64 (25, x3);                     \
-    x1 = ROTL64 (31, x1);                     \
-    x3 = x3 ^ x2 ^ RSHIFT64(3, x0); \
+    x3 = DROTL32 (25, x3);                     \
+    x1 = DROTL32 (31, x1);                     \
+    x3 = x3 ^ x2 ^ DRSHIFT32(3, x0); \
     x1 = x1 ^ x0 ^ x2;        \
-    x2 = ROTL64 (29, x2);                     \
-    x0 = ROTL64 (19, x0);                    \
+    x2 = DROTL32 (29, x2);                     \
+    x0 = DROTL32 (19, x0);                    \
   } while (0)
 
 #define ROUND64_INVERSE(which, subkey, x0,x1,x2,x3, y0,y1,y2,y3) \
@@ -461,7 +469,7 @@
 
 void
 serpent_decrypt (const struct serpent_ctx *ctx,
-		 unsigned length, uint8_t * dst, const uint8_t * src)
+		 size_t length, uint8_t * dst, const uint8_t * src)
 {
   assert( !(length % SERPENT_BLOCK_SIZE));
 
